﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;

namespace Airplane
{
    public partial class Airplane : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        
        uint m_background_music;
        uint m_propellor_sound;
        uint m_explosion_sound;
        
        uint m_ariel18;
        uint m_plane_object;
        bool m_plane_uses_x_file = true;
        uint m_bomb_object;
        uint m_explosion_object;


        double[,] m_world_bounding_box;
        
        double m_fog_visible_distance = 1500000;
        double m_fog_max_visible_distance = 1e8;
        double m_fog_min_visible_distance = 1e5;


        double m_plane_max_speed=2000;
        double m_plane_min_speed=100;
        double m_plane_speed = 900;
        int m_fire_rate = 300; //allow a bomb every 300 milliseconds. Change it for faster or lower fire rate

        double m_camera_min_height=2000; ////height above the ground
        double m_plane_min_height=6000; //height above the ground

        uint m_track1;
        uint m_enemy_plane1;
        uint m_enemy_tank1;

        
        public Airplane()
        {
            InitializeComponent();
        }

        private void Airplane_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
            //_3D.STATE_engine_close();
        }

        //the form loading event
        private void Form_Load(object sender, EventArgs e)
        {
            //Set Background music
            m_background_music = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Elgar.mp3", 0, _3D.SOUND_DISTANCE_DEFAULT);
            _3D.STATE_sound_play(m_background_music, _3D.SOUND_LOOP_NORMAL);
            m_explosion_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Explosion1.wav", 0, _3D.SOUND_DISTANCE_DEFAULT);
            
            
            m_ariel18 = _3D.STATE_font_create("Ariel", 18, _3D.FONT_CREATE_DEFAULT);
            _3D.STATE_font_write_text(m_ariel18, "Background music ON", 0, 40, 255, 50, 255, 255, _3D.FONT_WRITE_DEFAULT, 100);
            
            
            //load_world            
            load_world("..\\..\\..\\..\\worlds\\terrain2\\terrain2.3dstate");

            _3D.STATE_object_set_speed_units(_3D.UNITS_PER_RENDER);


            //one can create tracks using the worldbuilder/webmaker (this is the case here)
            //we can also create them from code or simply load them from a text file (see 3D Training/Models/Tracks/track1.txt)
            m_track1=_3D.STATE_track_get_using_name("track1");


            //load_airplane("..\\..\\..\\..\\models\\airplane\\airplane.x");

            m_plane_uses_x_file = false;
            load_airplane("..\\..\\..\\..\\models\\airplane\\f16.3dstate");

            load_bomb("..\\..\\..\\..\\models\\bombs\\oildrum.3dstate");

            load_explosion("..\\..\\..\\..\\models\\effects\\explosion.wld");

            add_enemy_plane();

            add_enemy_tank();


	        _3D.STATE_font_write_text(m_ariel18, "Keys: M, F1,  1,2,3,4,5,   Q,W,S,   A,S,  D,F,  G,H,   J,K   B, Space bar", 0, 20, 255, 80, 80, 0, _3D.FONT_WRITE_DEFAULT, 100);

	        //Just one line is needed to add the shadows to the plane !!!
	        _3D.STATE_object_auto_add_shadows(m_plane_object, _3D.ON, 1);
	        _3D.STATE_font_write_text(m_ariel18, "Shadow ON", 0, 35, 255, 255, 50, 255, _3D.FONT_WRITE_DEFAULT, 100);

	        //Set the sound of the wind/plane
	        m_propellor_sound=_3D.STATE_sound_load("..\\..\\..\\..\\sound\\propeller_sound.wav", m_plane_object, 600000);
	        _3D.STATE_sound_set_volume(m_propellor_sound, 800);
	        _3D.STATE_sound_play(m_propellor_sound, _3D.SOUND_LOOP_NORMAL);
	        _3D.STATE_font_write_text(m_ariel18, "Wind noise: ON", 0, 45, 255, 0, 255, 0, _3D.FONT_WRITE_DEFAULT, 100);

	
            make_the_camera_follow_the_airplane_automatically();

            _3D.STATE_engine_set_fog(_3D.ON, 220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);

           

            show_help_popup();

            //add our OnIdle function to the application on idle event
            //see more details at the Onidle function in this page
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;


            ActiveControl = pictureBox_rendering_window;

        }


        bool load_tank(string file_name)
        {
            //load the tank
            m_enemy_tank1 = _3D.STATE_object_create_from_file(file_name);
            if (m_enemy_tank1 == 0)
            {
                MessageBox.Show("Failed loading tank model.\r\nFile=" + file_name, "Failed loading tank model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            //The object is turned around so lets turn it 180 degrees
            //This is important so that the _3D.STATE_chase functions work as
            //we expect them to work.
            //should come immediatelly after we load the object
            _3D.STATE_object_rotate_z(m_enemy_tank1, 180, _3D.WORLD_SPACE);
            _3D.STATE_object_redefine_axis_system(m_enemy_tank1);

            double[] scale = { 10, 10, 10 }; //make the tank bigger
            _3D.STATE_object_set_scale(m_enemy_tank1, scale);

            return (true);

        }


        void add_enemy_tank()
        {
            load_tank("..\\..\\..\\..\\models\\tanks\\t72bm-tank_des.3dstate");

            _3D.STATE_object_set_chase_type(m_enemy_tank1, _3D.CHASE_TRACK);
            _3D.STATE_object_set_track(m_enemy_tank1, m_track1);

            double[] offset = new double[3] { 0, 0, 50000 };
            _3D.STATE_object_set_track_offset(m_enemy_tank1, offset);
            _3D.STATE_object_set_absolute_speed(m_enemy_tank1, m_plane_speed);

            //lets move the enemy object abit so it is not on the same spot as out own plane
            _3D.STATE_object_move(m_enemy_tank1, _3D.OBJECT_SPACE, -50000, 0, 0);

            
            double height, width, length;
            _3D.STATE_object_get_dimensions(m_enemy_tank1, out length, out width, out height);
            _3D.STATE_object_set_falling_from_track(m_enemy_tank1, height / 2, _3D.YES);


        }


        void add_enemy_plane()
        {

            m_enemy_plane1 = _3D.STATE_object_duplicate(m_plane_object, _3D.NO);

            _3D.STATE_object_set_chase_type(m_enemy_plane1, _3D.CHASE_TRACK_AIRPLANE);
            _3D.STATE_object_set_track(m_enemy_plane1, m_track1);

            double[] offset = new double[3] { 0, 0, 50000 };
            _3D.STATE_object_set_track_offset(m_enemy_plane1, offset);
            _3D.STATE_object_set_absolute_speed(m_enemy_plane1, m_plane_speed);

            //lets move the enemy object abit so it is not on the same spot as out own plane
            _3D.STATE_object_move(m_enemy_plane1, _3D.OBJECT_SPACE, -50000, 0, 0);

        }


        bool load_explosion(string file_name)
        {
            

            m_explosion_object = _3D.STATE_object_create_from_file(file_name);
            if (m_explosion_object == 0)
            {
                MessageBox.Show("Failed loading explosion model.\r\nFile=" + file_name, "Failed loading airplane model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            double[] scale = { 10000, 10000, 10000 }; //make the plane bigger
            _3D.STATE_object_set_scale(m_explosion_object, scale);


            _3D.STATE_object_disable(m_explosion_object);
            return (true);



        }


        bool load_bomb(string file_name)
        {
            m_bomb_object = _3D.STATE_object_create_from_file(file_name);
            if (m_bomb_object == 0)
            {
                MessageBox.Show("Failed loading bomb model.\r\nFile=" + file_name, "Failed loading airplane model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            double[] scale = { 100, 100, 100 }; //make the plane bigger
            _3D.STATE_object_set_scale(m_bomb_object, scale);



            _3D.STATE_object_disable(m_bomb_object);
            return (true);


        }

        
        
        bool load_airplane(string file_name)
        {
	        //load the airplane
	        m_plane_object=_3D.STATE_object_create_from_file(file_name);
            if (m_plane_object == 0)
            {
                MessageBox.Show("Failed loading airplane model.\r\nFile=" + file_name, "Failed loading airplane model", MessageBoxButtons.OK, MessageBoxIcon.Warning);                
                return (false); //safty
            }


            //The object is turned around so lets turn it 180 degrees
            //This is important so that the _3D.STATE_chase functions work as
            //we expect them to work
            if (m_plane_uses_x_file)
            {
                _3D.STATE_object_rotate_z(m_plane_object, 180, _3D.WORLD_SPACE);
                _3D.STATE_object_redefine_axis_system(m_plane_object);

                double[] scale = { 1000, 1000, 1000 }; //make the plane bigger
                _3D.STATE_object_set_scale(m_plane_object, scale);
            }

            else
            {
                double[] scale = { 10, 10, 10 }; //make the plane bigger
                _3D.STATE_object_set_scale(m_plane_object, scale);
            }



            return (true);

        }

        bool load_world(string world_file_name)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }

            //Get the world dimesions so that we can watch later
            //that the plane doesn't get out of the world ...
            m_world_bounding_box = new double[2, 3];
            _3D.STATE_group_get_bounding_box(0, m_world_bounding_box);

            double x_width = m_world_bounding_box[1, 0] - m_world_bounding_box[0, 0];
            double y_width = m_world_bounding_box[1, 1] - m_world_bounding_box[0, 1];
            double max_side = Math.Max(x_width, y_width);

            uint camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_distance_from_eye(camera, max_side/4000);


            //Put some ground details texture
            uint ground_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\terrain1\\Bitmaps\\second_bitmap.bmp", -1);
            if (ground_bmp != 0)
            {
                //Try cancel this line or putting a different number than 100 and see what it does
                _3D.STATE_group_wrap_a_bitmap(0, ground_bmp, 35, 35, 1);
            }

            
          
            add_some_light();

            return (true);


        }


        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            _3D.STATE_3D_card_set_texture_tiling_mode(_3D.TEXTURE_TILING_WRAP);


            move_plane();

            check_whether_a_bomb_is_fired();


            check_whether_a_bomb_has_to_explode();

            change_settings_according_to_keyboard_input();

            make_sure_we_get_more_OnIdle_events();
        }

        void move_plane()
        {
            _3D.STATE_object_move(m_plane_object, _3D.OBJECT_SPACE, -m_plane_speed, 0, 0); //move the plane forwad. This is towards the plane negative X axis

            turn_plane_according_to_keyboard_input(m_plane_object);

            m_plane_speed = change_speed_according_to_keyboard_input(m_plane_speed, m_ariel18);

            check_that_the_plane_doesnt_go_out_of_the_terrain();

            make_sure_the_plane_doesnt_go_below_the_ground(m_plane_object, m_ariel18);

            check_that_the_camera_doesnt_go_under_the_ground();

        }


        void make_sure_we_get_more_OnIdle_events()
        {
            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...
        }


        private void pictureBox_rendering_window_Paint(object sender, PaintEventArgs e)
        {
            OnIdle(null, null);
        }




    }
}
