﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;
using Math3D;


namespace Airplane
{
    public partial class Airplane : Form
    {

        List<uint> m_bombs = new List<uint>();


        void show_help_popup()
        {
            MessageBox.Show("Instructions:\n\nPress M to Fire\r\n\r\nNote that there is an improved version of this demo in the 3D Networking SDK.\r\n\r\nPress F1 To get this help popup\n\nUse the arrow keys to navigate the plane\n\nUse Q,W,E to change the camera modes\n\nUse 1,2,3,4,5 to control the camera distance\n\nUse A,S to toggle shadow modes\n\nUse D,F,G,H to toggle sound modes\n\nUse J,K to control fog intensity\n\nUse the space bar and 'B' to increase or decrease the speed\n\nThings to notice\n------------------\n1. Press E (NO_CHASE) and notice how the sound gets automatically stronger when the plane gets closer to you\nNotice that when the plane is on the right, you hear it more from the right speaker\n(make sure your speaker are positioned correctly)\n\n2. Press W and then 2 and notice the shadows on the plane's wing and tail (when turning)\n", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        void add_some_light()
        {
            //Add a light source
            double[] location = { 0, 0, 10000 }; //put it up in the sky like the sun
            uint light = _3D.STATE_light_create("light1", location);
            _3D.STATE_light_set_type_of_light(light, _3D.LIGHT_AMBIENT | _3D.LIGHT_DIFFUSE);
            _3D.STATE_light_set_ambient(light, 0.8);
            _3D.STATE_light_set_specular(light, 0);
            _3D.STATE_light_set_diffuse(light, 3);
            _3D.STATE_light_activate(light, _3D.LIGHT_SET);

            location[2] = 10000000; //this height will look better with the plane shadows since is simulate the sun and the sun is very high above ...
            _3D.STATE_light_set_location(light, location);


        }


        void make_the_camera_follow_the_airplane_automatically()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_chase_type(camera, _3D.CHASE_FLEXIBLE);
            _3D.STATE_camera_set_object_to_chase(camera, m_plane_object);
            _3D.STATE_camera_set_chase_distance(camera, 40000); //set the camera behind the airplane
            double[] offset = { 0, 0, 10000 }; //set an offset so it is always a bit above
            _3D.STATE_camera_set_chase_offset(camera, offset);
        }



        void check_that_the_camera_doesnt_go_under_the_ground()
        {
            uint camera = _3D.STATE_camera_get_default_camera();

            double height;
            uint ground_poly = _3D.STATE_camera_get_height_above_ground(camera, 0, out height);
            if (ground_poly != 0)
            {
                if (height < m_camera_min_height)
                {
                    _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, m_camera_min_height - height);
                }
            }

        }

        void explode_bomb(uint bomb)
        {
            uint explosion = _3D.STATE_object_duplicate(m_explosion_object, _3D.NO);

            double[] location = new double[3];
            _3D.STATE_object_get_location1(bomb, location);

            _3D.STATE_object_set_location1(explosion, location);

            double length, width, height;
            _3D.STATE_object_get_dimensions(explosion, out length, out width, out height);
            _3D.STATE_object_move(explosion, _3D.WORLD_SPACE, 0,0, height/2);


            _3D.STATE_object_delete(bomb);

            _3D.STATE_object_set_event(explosion, 5000, _3D.STATE_DELETE);

            _3D.STATE_sound_play(m_explosion_sound, _3D.SOUND_NO_LOOP);

        }

        void check_whether_a_bomb_has_to_explode()
        {
            List<uint> bombs_to_delete = new List<uint>();
            
            foreach(uint bomb in m_bombs)
            {
                double height_above_ground=_3D.STATE_object_get_center_height_above_ground(bomb, _3D.YES);
                if (height_above_ground>0)
                    continue;

                explode_bomb(bomb);
                bombs_to_delete.Add(bomb);

            }

            foreach(uint bomb in bombs_to_delete)
                m_bombs.Remove(bomb);


        }

        DateTime m_last_time_we_fired = new DateTime(1980, 1, 1);

        //allow firing only every m_fire_rate milliseconds
        //we can change it to even 1 millisecond for crazy fire rate
        bool can_we_fire_again()
        {
            DateTime now = DateTime.Now;

            TimeSpan time_past_from_last_bomb = now - m_last_time_we_fired;

            if (time_past_from_last_bomb.TotalMilliseconds < m_fire_rate)
                return(false);

            return (true);

        }
       
        void check_whether_a_bomb_is_fired()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.M) >= 0)
                return;

            if (can_we_fire_again() == false)
                return;

            m_last_time_we_fired = DateTime.Now;

            uint bomb = _3D.STATE_object_duplicate(m_bomb_object, _3D.YES);
            
            _3D.STATE_object_cancel_collision_test_for_chase_physics(bomb);
            
            double[] location=new double[3];
            _3D.STATE_object_get_location1(m_plane_object, location);

            //location[2] -= 5000;
            _3D.STATE_object_set_location1(bomb, location);

            _3D.STATE_object_set_chase_type(bomb, _3D.CHASE_PHYSICS);
            double[] direction = new double[3];
            _3D.STATE_object_get_direction1(m_plane_object, direction);

            _3D.STATE_object_set_speed(bomb, direction);
            _3D.STATE_object_set_absolute_speed(bomb, m_plane_speed*50);

            double[] force= {0,0, -30};
            _3D.STATE_object_set_force(bomb, force);

            _3D.STATE_object_set_friction(bomb, 0.01);

            _3D.STATE_object_set_elasticity(bomb, 0);
            
            
            _3D.STATE_object_enable(bomb);

            m_bombs.Add(bomb);


        }

        void change_settings_according_to_keyboard_input()
        {
            //Show the help popup
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.F1) < 0)
                show_help_popup();

            uint camera = _3D.STATE_camera_get_default_camera();
            change_camera_properties_according_to_keyboard_input(camera);

            change_shadow_settings_according_to_keyboard_input(m_plane_object, m_ariel18);

            change_sound_settings_according_to_keyboard_input(m_background_music, m_propellor_sound, m_ariel18);

            change_fog_settings_according_to_keyboard_input();

        }
        
        
        void turn_plane_according_to_keyboard_input(uint plane_object)
        {
            //note that rotation around our airplane Y axis increases the tilt angle (how much the nose is higher or lower than the tail)
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0) _3D.STATE_object_rotate_y(plane_object, 1, _3D.OBJECT_SPACE);
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0) _3D.STATE_object_rotate_y(plane_object, -1, _3D.OBJECT_SPACE);

            //note tat rotation around the airplane X axis increases the bank angle (when one wing is higher and the other wind is lower)
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0) _3D.STATE_object_rotate_x(plane_object, 1, _3D.OBJECT_SPACE);
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0) _3D.STATE_object_rotate_x(plane_object, -1, _3D.OBJECT_SPACE);

            double bank = _3D.STATE_object_get_bank(plane_object);

            //Make the turn proportional to the bank angle. The more one wing is higher and the other one is lower the turn
            //will be bigger
            _3D.STATE_object_rotate_z(plane_object, -bank * 0.02, _3D.OBJECT_SPACE);

            //decrease the bank == try to straighten the plane
            bank *= 0.98;
            _3D.STATE_object_set_bank(plane_object, bank);

            //try to reduce the tilt and thus straighten the plane (so that the nose is at the same hight as the tail)
            double tilt = _3D.STATE_object_get_tilt(plane_object);
            tilt *= 0.9;
            _3D.STATE_object_set_tilt(plane_object, tilt);

        }

        double change_speed_according_to_keyboard_input(double speed, uint font_handle)
        {
            double prev_speed = speed;
            if(_3D.STATE_utilities_get_keyboard_key_state((int)' ')<0)
		        speed+=1;

	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'B')<0)
		        speed-=1;

	        if(speed>m_plane_max_speed)
		        speed=m_plane_max_speed;

	        if(speed<m_plane_min_speed)
		        speed=m_plane_min_speed;

	        //Write speed to screen
            if (prev_speed != speed)
            {
                string speed_str = "Speed: " + ((int)speed).ToString();
                _3D.STATE_font_write_text(font_handle, speed_str, 0, 25, 255, 255, 255, 120, _3D.FONT_WRITE_DEFAULT, 100);
            }


	        return(speed);
        }


        int m_number_of_possible_crashes=0;

        void check_that_the_plane_doesnt_go_out_of_the_terrain()
        {
            double[] location = new double[3];
            _3D.STATE_object_get_location1(m_plane_object, location);
            for (int i = 0; i < 2; i++)
            { //note that <2 and not <3 because we don't care about getting out of the world on the X axis
                if ((location[i] > m_world_bounding_box[1, i] * 0.9) || (location[i] < m_world_bounding_box[0, i] * 0.9))
                {
                    _3D.STATE_object_set_location(m_plane_object, 0, 0, 5000);
                }
            }

        }


        
        int make_sure_the_plane_doesnt_go_below_the_ground(uint plane_object, uint font_handle)
        {
	         	
        	
	        //Make sure the plane doesn't get into the ground
	        double max_height, min_height;
                
            _3D.STATE_object_get_bounding_box_height_above_ground(plane_object, _3D.YES, out min_height, out max_height);

	        if(min_height<m_plane_min_height) {
		        _3D.STATE_object_move(plane_object, _3D.WORLD_SPACE, 0,0, m_plane_min_height-min_height);
		        m_number_of_possible_crashes++;
		        //speed*=0.98; //slow down


		        //string crashes_text= "Possible Crashes: " + m_number_of_possible_crashes;
		        //_3D.STATE_font_write_text(font_handle, crashes_text, 0, 30, 255, 255, 255, 120, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }

	        return(m_number_of_possible_crashes);
        }


        void change_camera_properties_according_to_keyboard_input(uint camera)
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)'1') < 0)
                _3D.STATE_camera_set_chase_distance(camera, 15000); //set the camera behind the airplane

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'2') < 0)
                _3D.STATE_camera_set_chase_distance(camera, 30000); //set the camera behind the airplane

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'3') < 0)
                _3D.STATE_camera_set_chase_distance(camera, 45000); //set the camera behind the airplane

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'4') < 0)
                _3D.STATE_camera_set_chase_distance(camera, 60000); //set the camera behind the airplane

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'5') < 0)
                _3D.STATE_camera_set_chase_distance(camera, 75000); //set the camera behind the airplane


            if (_3D.STATE_utilities_get_keyboard_key_state((int)'Q') < 0)
                _3D.STATE_camera_set_chase_type(camera, _3D.CHASE_FLEXIBLE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'W') < 0)
                _3D.STATE_camera_set_chase_type(camera, _3D.CHASE_PRECISE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)'E') < 0)
                _3D.STATE_camera_set_chase_type(camera, _3D.NO_CHASE);

        }


        void change_shadow_settings_according_to_keyboard_input(uint plane_object, uint ariel18)
        {

	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'A')<0) {
		        _3D.STATE_object_auto_add_shadows(plane_object, _3D.ON, 1);
		        _3D.STATE_font_write_text(ariel18, "Shadow ON", 0, 35, 255, 255, 50, 255, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }

	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'S')<0) {
		        _3D.STATE_object_auto_add_shadows(plane_object, _3D.OFF, 1);
		        _3D.STATE_font_write_text(ariel18, "Shadow OFF", 0, 35, 255, 255, 50, 255, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }
        }


        void change_sound_settings_according_to_keyboard_input(uint background_music, uint propellor_sound, uint ariel18)
        {
	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'D')<0) {
		        if(_3D.STATE_sound_is_sound_playing(background_music)==_3D.NO)
			        _3D.STATE_sound_play(background_music, _3D.SOUND_LOOP_NORMAL);
		        _3D.STATE_font_write_text(ariel18, "Background music ON", 0, 40, 255, 50, 255, 255, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }

	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'F')<0) {
		        if(_3D.STATE_sound_is_sound_playing(background_music)==_3D.YES)
			        _3D.STATE_sound_stop(background_music);
        		
		        _3D.STATE_font_write_text(ariel18, "Background music OFF", 0, 40, 255, 50, 255, 255, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }


	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'G')<0) {
		        if(_3D.STATE_sound_is_sound_playing(propellor_sound)==_3D.NO)
			        _3D.STATE_sound_play(propellor_sound, _3D.SOUND_LOOP_NORMAL);
		        _3D.STATE_font_write_text(ariel18, "Wind noise: ON", 0, 45, 255, 0, 255, 0, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }

	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'H')<0) {
		        if(_3D.STATE_sound_is_sound_playing(propellor_sound)==_3D.YES)
			        _3D.STATE_sound_stop(propellor_sound);
        		
		        _3D.STATE_font_write_text(ariel18, "Wind noise: OFF", 0, 45, 255, 0, 255, 0, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
	        }
        }


        //decrease or increase the fog intensity according to the keyboard input
        void change_fog_settings_according_to_keyboard_input()
        {
	        double factor=1;
	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'J')<0) {
		        factor=0.95;
	        }
	        if(_3D.STATE_utilities_get_keyboard_key_state((int)'K')<0) {
		        factor=1.05;
	        }

	        if(factor==1)
		        return; //no change is needed. Neither of the keys is pressed

	        m_fog_visible_distance*=factor;

        	
	        if(m_fog_visible_distance<m_fog_min_visible_distance)
		        m_fog_visible_distance=m_fog_min_visible_distance;

       	
	        if(m_fog_visible_distance>m_fog_max_visible_distance) {
		        m_fog_visible_distance=m_fog_max_visible_distance;
		        _3D.STATE_engine_set_fog(_3D.OFF,  220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance,  _3D.FOG_START_DISTANCE_DEFAULT, 0);
		        _3D.STATE_font_write_text(m_ariel18, "Fog: OFF", 0, 50, 255, 0, 0, 100, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);
		        return;
	        }
        	
        	
	        _3D.STATE_engine_set_fog(_3D.ON,   220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);

            string fog_text = "Fog intensity: " + (int)m_fog_visible_distance;
	        _3D.STATE_font_write_text(m_ariel18, fog_text, 0, 50, 255, 0, 0, 100, _3D.FONT_WRITE_DEFAULT, Int32.MaxValue);

        }

    
    }

}