﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;

// AI == Artificial Inteligence
//
// From level 1 (0 is the first level) of the game, the enemy plane dont just go around their track without any thinking.
// Here we have the code that will put some wizdom in the enemy's plane movements
// We will continue to use tracks only that we will modify the track points values and thus the 
// plane will follow the modified track.
//
// since in level 1 and up we change the track points in order to give some wizdom to the enemy planes
// We need that every plane will have its own track (in level one, only one track is used for all palnes, each time with different offset)
//
namespace OneAgainst100Planes
{

    public enum AI { None, Plane, Insect }; //insect and enemy planes have different method of attacking
    //the insect home on the target and stick to the target once it reaches it
    //the plane fly pass the target.
    
    
    public partial class Game : Form
    {
        Random m_random = new Random(DateTime.Now.Millisecond);

        int m_attack_radius = 30000;

        double m_seconds_between_enemy_attacks = 2;
        DateTime m_last_time_enemy_attack=new DateTime(1980,1,1);

        void set_enemy_attack_frequency()
        {
            m_seconds_between_enemy_attacks = 20 / m_enemy_planes.Count;
            if (m_enemy_planes.Count <= 0)
                return; //safty

            
            if (m_current_level.enemy_shooting_style == EnemyShootingStyle.AimOnPlayer)
                m_seconds_between_enemy_attacks = 120 / m_enemy_planes.Count;
            
        }

        void make_sure_enemy_planes_dont_go_under_the_ground(List<uint> enemy_planes)
        {
            double[] camera_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, camera_location);
            
            //checking collision with the terrain is expensive so we will try to do it just when we really have to
            foreach (uint enemy in enemy_planes)
            {
                double[] location = new double[3];
                _3D.STATE_object_get_location1(enemy, location);
                if (location[2] > m_highest_mountain_top)
                    continue; //it is high in the air so no need to check collision

                //the enemy planes use tracks. the points of the track are always on the ground
                //so basically they cant go undergraound too much
                //sometimes though when there is a hill between two points of the track the planes will fly
                //through the hill. so we have to watch from those situations.

                //lets check that the camera is in the area of the enemy plane, because if it is far away
                //it doesnt matter if it goes a little bit undertheground cause anyhow we are not going to see it.

                double square_distance=Math3D.Vector.square_distance_between_points(location, camera_location);
                if (square_distance > 1e12)
                    continue; //too far away to bother

                make_sure_enemy_plane_doesnt_go_below_the_ground(enemy);
            }
        }


        void make_sure_enemy_plane_doesnt_go_below_the_ground(uint plane_object)
        {
            double[] offset = { 0, 0, 0 };
            
            //Make sure the plane doesn't get into the ground
            double height_above_ground = _3D.STATE_object_get_center_height_above_ground(plane_object, _3D.YES);

            double height, width, length;
            _3D.STATE_object_get_dimensions(plane_object, out length, out width, out height);

            double permited_height_above_ground = 2*m_plane_min_height + height / 2;  //the permited height is from the plane center. we add the height/2 so it is from the plane bottom
                                                                                      //2*m_plane_min_height is simply because we want a bigger height above the ground then the default for the player plane


            if (height_above_ground < permited_height_above_ground)
            {
                offset[2] = 2 * permited_height_above_ground - height_above_ground;
                _3D.STATE_object_set_track_offset(plane_object, offset); //this will move the track up so the enemy wont go so down

                //_3D.STATE_object_move(plane_object, _3D.WORLD_SPACE, 0, 0, m_plane_min_height - height_above_ground);
            }

        }


        void manage_enemy_attacks(List<uint> enemies, AI enemy_AI)
        {
            if (enemy_AI==AI.None)
                return;

            if (m_game_status == GameStatus.Level_Is_Finished)
                return; //let the user read the instructions for the next level without the enemies buzzing in front of the camera ...
            
            DateTime now = DateTime.Now;
            TimeSpan time_passed = now - m_last_time_enemy_attack;
            //if (time_passed.TotalMilliseconds < m_current_level.enemy_firing_delay)
            if (time_passed.TotalSeconds < m_seconds_between_enemy_attacks)
                return;

            if (enemies.Count <= 0)
                return;

            m_last_time_enemy_attack = now;

            //ok, an enemy needs to attack. So lets pick an enmy plane by random.

            int enemy_index = m_random.Next(0, enemies.Count - 1);

            enemy_attack(enemies[enemy_index]);

        }

        //when a plane attacks, it dives towards its target and then continues on the same line (but going up )
        //The enemy plane start its dive from his current location, then it reaches the plane location,
        //and now we want another point which is the continuation of the line
        //
        //return the X,Y,Z of a point that continues the line/segemnt start_xyz - end_xyz.
        //the point we return is 1 million units from end_xyz
        double[] get_point_on_the_same_line(double[] start_xyz, double[] end_xyz, double distance_from_end)
        {
            double[] vec = Math3D.Vector.copy(end_xyz);
            Math3D.Vector.sub_vec(vec, start_xyz);
            vec[2] = 0; //make it a horizontal vector
            int rc=_3D.STATE_math_normalize_vector(vec);
            if (rc == _3D.VR_ERROR)
                Math3D.Vector.set(vec, 1, 0, 0);
            
            Math3D.Vector.mul_vec(vec, 1e6); // make a size of 1 million
            Math3D.Vector.add_vec(vec, end_xyz);
            
            return (vec);
        }


        int track_get_next_point_index(uint track, int current_index)
        {
            int num_of_points = _3D.STATE_track_get_number_of_points(track);
            int next_point_index = current_index+1;
            if (next_point_index < num_of_points)
                return (next_point_index);

            return (0);
        }

        void add_a_random_offset(double[] player_plane_location)
        {
            int x = m_random.Next(-m_attack_radius, m_attack_radius);
            int y = m_random.Next(-m_attack_radius, m_attack_radius);
            //int z = m_random.Next(-m_attack_radius, m_attack_radius);

            player_plane_location[0] += x;
            player_plane_location[1] += y;
            //player_plane_location[2] += z;

        }

        void set_next_point_on_track_to_be_equal_to_player_plane_location(uint enemy, uint track, int next_point_index, double[] player_plane_location)
        {
            add_a_random_offset(player_plane_location);
            _3D.STATE_track_set_point(track, next_point_index, player_plane_location);
        }


        //The enemies always shoot in the direction of their movement
        //so if we set the next point on the track to be the location of the player
        //then the enemy plane will fly strghit towords the player and
        // will shoot at the player !!!
        // We dont want the enmy plane to shoot exactly at the player since the player will move a bit forward and so the missile will miss
        //therefor we need to add some random factor. For that we use m_attack_radius
        void enemy_attack(uint enemy)
        {
            double[] offset = { 0, 0, 0 };
            _3D.STATE_object_set_track_offset(enemy, offset); //reset the track offset so that the enemy can home directly on the player plane
                                                              //we use the offset to avoid collision with the ground when the track intersect with the terrain (see make_sure_enemy_plane_doesnt_go_below_the_ground() )
            
            
            uint track = _3D.STATE_object_get_track_handle(enemy);
            int next_point_index = _3D.STATE_object_get_next_point_on_track(enemy);

            double[] player_plane_location = new double[3];
            _3D.STATE_object_get_location1(m_plane_object, player_plane_location);
            double player_plane_height = player_plane_location[2];

            //note that this function also changes player_plane_location[] it adds some offset
            set_next_point_on_track_to_be_equal_to_player_plane_location(enemy, track, next_point_index, player_plane_location);

            if (m_levels[m_game_level].enemy_AI == AI.Insect)
                return; //for insects AI this is enough

            //for plane attacking we set another point so that the plane continues flying
            int next_next_point_index = track_get_next_point_index(track, next_point_index);

            double[] enemy_location = new double[3];
            _3D.STATE_object_get_location1(enemy, enemy_location);

            double[] next_next_point_xyz=get_point_on_the_same_line(enemy_location, player_plane_location, 1000000);
            next_next_point_xyz[2] = player_plane_height + 100000;

            _3D.STATE_track_set_point(track, next_next_point_index, next_next_point_xyz);
            

        }
        
        
        
        uint track_create_copy(uint obj, uint trk)
        {
            int num_of_points = _3D.STATE_track_get_number_of_points(trk);
            double[] offset = new double[3];
            _3D.STATE_object_get_track_offset(obj, offset);

            uint track_copy = _3D.STATE_track_create("trk", _3D.YES, null, 0);


            for (int i = 0; i < num_of_points; i++)
            {
                double[] pnt = new double[3];
                _3D.STATE_track_get_point(trk, i, pnt);

                pnt[0] += offset[0];
                pnt[1] += offset[1];
                pnt[2] += offset[2];

                _3D.STATE_track_add_point_to_end(track_copy, pnt);
            }

            return (track_copy);

        }

        
        void enemy_create_own_track(uint enemy)
        {
            uint current_track = _3D.STATE_object_get_track_handle(enemy);
            if (current_track == 0)
            {
                MessageBox.Show("internal error. object has no track");
                return;
            }

            uint new_track = track_create_copy(enemy, current_track);
            _3D.STATE_object_set_track(enemy, new_track);
            _3D.STATE_object_set_track_offset(enemy, new double[] { 0, 0, 0 });

        }
    
    }

}