﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;



namespace OneAgainst100Planes
{
    public enum ModelIndex { Piper = 0, Dino = 1, Fairy = 2, F16 = 3, Missile1 = 4, Bat = 5, Dragonfly = 6, Fly = 7, Bomb = 8, Explosion = 9, Tank = 10, TankDestroyed = 11, A10 = 12, F111 = 13, Junkers52 = 14, Junkers88 = 15, Ling = 16, Mig29 = 17, Blackbird=18, Stealth=19, SU37=20,  None = 1000 };

    static public class AllModels
    {
        static public Model[] models = { 
                             new Model("airplane\\airplane.x", 2000, 180, ObjectType.Plane,ModelIndex.Missile1,"plane2.wav", null), 
                             new Model("enemies\\dino\\pterano.3dstate", 6, -90, ObjectType.Plane,ModelIndex.Missile1,"hawk.wav", "monster_dead1.wav"),
                             new Model("enemies\\Fairy\\Faerie.md2","enemies\\Fairy\\Faerie_3d.bmp" , "stand" ,300, 0,-90 ,ObjectType.Plane,ModelIndex.None,"hawk2.wav", "monster_dead6.wav"),
                             new Model("airplane\\f16.3dstate", 16, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("missiles\\rocket.X", 100, -90, ObjectType.Missile,ModelIndex.None, null, null),
                             new Model("enemies\\Bat\\Bat.3dstate", 3, 90, -50, ObjectType.Plane,ModelIndex.Missile1,"bat1.wav", "monster3.wav"),
                             new Model("enemies\\DragonFly\\Dragonfly.3dstate", 6, 0, ObjectType.Plane,ModelIndex.Missile1,"bee2.wav", "monster1.wav"),
                             new Model("enemies\\Fly\\fly.3dstate", 3, 0 , ObjectType.Plane,ModelIndex.Missile1,"fly.wav", "monster2.wav"),
                             new Model("bombs\\oildrum.3dstate", 100, 0, ObjectType.Bomb,ModelIndex.None,null,null),
                             new Model("effects\\explosion.wld", 10000, 0, ObjectType.Unknown,ModelIndex.None,null,null),
                             new Model("tanks\\t72bm-tank_des.3dstate", 10, 180, ObjectType.Tank,ModelIndex.None,null,null),
                             new Model("tanks\\t72bm-tank_des_X1.3dstate", 10, 180, ObjectType.Tank,ModelIndex.None,null,null),
                             new Model("airplane\\A-10thunderbolt1.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\f111.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\junkers52.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\junkers88.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\ling.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\mig29.3dstate", 10, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\SR-17blackbird.3dstate", 15, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\Stealth.3dstate", 20, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null),
                             new Model("airplane\\su37-flanker-fighter_rus.3dstate", 15, 0, ObjectType.Plane,ModelIndex.Missile1,"jet.wav", null)
                           };


    }


    //The Model class helps us load one master 3D object.
    //this object is our master key.
    //for example we have only one model of a piper plane. we might have 100 piper planes which are all duplicates of this one model
    //it also contain some data such as sounds that will be used by all the duplication of the master object
    //There is always only one copy of each type of model.
    public class Model
    {
        public uint object_handle;

        private string file_name;
        private string skin = null;
        private string sequence_name;
        private double scale;
        private double initial_turn_z = 0;
        private double initial_turn_y = 0;
        private ObjectType model_type;

        public ModelIndex missile_index; //The missile/bomb that goes with this model
        string sound_file; //the model sound
        const int number_of_simultaneous_sounds = 5; //meaning we will hear the closest 5 enemy planes 
        public uint[] model_sound_handles = new uint[number_of_simultaneous_sounds]; //Even if we have 100 objects which are all duplicates of one model
        //we only use 5 sounds simultanouesly. (the 5 closest to the camera)

        string death_sound_file; //the sound that is played when the object is killed
        uint death_sound_handle;


        public Model(string file, double sc, double turn_z, ObjectType mdl_type, ModelIndex missile, string sound_file_name, string death_sound)
        {
            file_name = file;
            scale = sc;
            initial_turn_z = turn_z;
            model_type = mdl_type;
            sound_file = sound_file_name;
            death_sound_file = death_sound;
            missile_index = missile;
        }

        public Model(string file, double sc, double turn_z, double turn_y, ObjectType mdl_type, ModelIndex missile, string sound_file_name, string death_sound)
        {
            file_name = file;
            scale = sc;
            initial_turn_z = turn_z;
            initial_turn_y = turn_y;
            model_type = mdl_type;
            sound_file = sound_file_name;
            death_sound_file = death_sound;
            missile_index = missile;

        }


        public Model(string file, string skin_file, string sequenceName, double sc, double turn_z, double turn_y, ObjectType mdl_type, ModelIndex missile, string sound_file_name, string death_sound)
        {
            file_name = file;
            skin = skin_file;
            sequence_name = sequenceName;
            scale = sc;
            initial_turn_z = turn_z;
            initial_turn_y = turn_y;
            model_type = mdl_type;
            sound_file = sound_file_name;
            death_sound_file = death_sound;
            missile_index = missile;

        }


        public void enable(bool is_enabled)
        {
            if (object_handle == 0)
                return;

            if (is_enabled)
                _3D.STATE_object_enable(object_handle);
            else
                _3D.STATE_object_disable(object_handle);

        }

        void set_animation_sequence(uint obj, string sequence_name)
        {
            uint animation = _3D.STATE_object_get_3D_animation(object_handle);
            if (animation == 0)
                return; //this object is not a 3d animation

            uint sequence = _3D.STATE_3D_sequence_get_using_name(animation, sequence_name);
            if (sequence == 0)
                return;

            _3D.STATE_object_set_3D_sequence(object_handle, sequence, 0);

        }

        void set_skin(uint obj, string skin_file)
        {
            string full_path = "..\\..\\..\\..\\models\\" + skin_file;

            uint bmp = _3D.STATE_bitmap_load(full_path, -1);
            if (bmp == 0)
            {
                MessageBox.Show("Failed loading skin: " + skin);
                return;
            }

            _3D.STATE_object_set_bitmap(object_handle, bmp, 0);
        }

        public void load(bool is_enabled)
        {
            if (object_handle != 0)
            {
                enable(is_enabled);
                return; //already loaded
            }

            object_handle = load(file_name, scale, initial_turn_z, initial_turn_y, model_type);

            if (object_handle == 0)
            {
                MessageBox.Show("Failed loading object: " + file_name);
                return;
            }

            if (file_name.IndexOf(".md") >= 0) //it is an md2 or md3 animation file. for example farie.md2
                set_animation_sequence(object_handle, sequence_name);

            if (skin != null)
                set_skin(object_handle, skin);

            enable(is_enabled);

            Game.m_total_number_of_object_created++;
            string name;
            _3D.STATE_object_get_name(object_handle, out name);
            Game.m_allocated_3d_objects_dict.Add(object_handle, "Master:" + name);

        }

        private static uint load(string file_name, double scale, double turn_around_z, double turn_around_y, ObjectType obj_type)
        {

            string full_path = "..\\..\\..\\..\\models\\" + file_name;

            uint obj = _3D.STATE_object_create_from_file(full_path);
            if (obj == 0)
            {
                MessageBox.Show("Failed loading model.\r\nFile=" + full_path, "Failed loading model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (0); //safty
            }

            //does nothing, for our own use later
            _3D.STATE_object_set_object_type_number(obj, (int)obj_type);

            if (turn_around_z != 0)
            {
                //The object is turned around so lets turn it 180 degrees
                //This is important so that the _3D.STATE_chase functions work as
                //we expect them to work
                _3D.STATE_object_rotate_z(obj, turn_around_z, _3D.WORLD_SPACE);
                _3D.STATE_object_redefine_axis_system(obj);
            }

            if (turn_around_y != 0)
            {
                _3D.STATE_object_rotate_y(obj, turn_around_y, _3D.WORLD_SPACE);
                _3D.STATE_object_redefine_axis_system(obj);
            }

            double[] scale_vec = { scale, scale, scale }; //make the object bigger
            _3D.STATE_object_set_scale(obj, scale_vec);

            return (obj);

        }

        public void increase_size(double percentage)
        {
            if (object_handle == 0)
                return; //safty

            double new_scale = scale * percentage / 100;
            double[] scale_vec = { new_scale, new_scale, new_scale };
            _3D.STATE_object_set_scale(object_handle, scale_vec);
        }

        //the obj is not important because later we attach the sound to the closest object
        public void load_sounds(uint obj)
        {
            //load death sound
            if (death_sound_handle == 0)
            {
                death_sound_handle = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\" + death_sound_file, obj, 600000);
                _3D.STATE_sound_set_volume(death_sound_handle, 1000);
            }
            else _3D.STATE_sound_stop(death_sound_handle);


            for (int i = 0; i < model_sound_handles.Length; i++)
            {
                if (model_sound_handles[i] != 0)
                {
                    _3D.STATE_sound_stop(model_sound_handles[i]);
                    continue;
                }

                model_sound_handles[i] = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\" + sound_file, obj, 600000);
                _3D.STATE_sound_set_volume(model_sound_handles[i], 500);

            }

        }

        public void stop_sounds()
        {
            foreach (uint sound in model_sound_handles)
            {
                if (sound == 0)
                    continue;

                _3D.STATE_sound_stop(sound);
            }

        }

        public void play_death_sound(uint obj)
        {
            uint current_playing_sound = _3D.STATE_sound_get_handle_using_entity(obj);
            if (current_playing_sound != 0)
                _3D.STATE_sound_stop(current_playing_sound);


            if (death_sound_handle == 0)
                return;

            _3D.STATE_sound_attach(death_sound_handle, obj);
            _3D.STATE_sound_play(death_sound_handle, _3D.SOUND_NO_LOOP);

        }


    }

}