﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;

namespace Alpha_Transparency
{
    public partial class AlphaTransparency : Form
    {

        //Decrease the alpha value of all pixels that their color intensity is less than the given threshold
        //The result is that the white colored man in the middle of the 3DSTATE logo bitmap is left untouched
        //(since the white color is brighter than the given threshold)
        unsafe bool remove_texture_exterior(uint bmp, byte threshold)
        {
            bool finished_working_flag = true;

            byte* mem_ptr;
            int number_of_bytes_per_pixel;
            int number_of_pixels_in_one_line;
            int number_of_lines;

            _3D.STATE_bitmap_start_direct_memory_access(bmp, 0, &mem_ptr, out number_of_bytes_per_pixel, out number_of_pixels_in_one_line,  out number_of_lines);
            if (mem_ptr == null)
            {
                //error_msg();
                return (true);
            }

            if (number_of_bytes_per_pixel != 4) //safty check
                return (true); //Since we called _3D.STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8);
            //at the beginning of the program, all non-transparent bitmaps must have been converted to 32 bit format

            int index = 0;
            for (int line = 0; line < number_of_lines; line++)
            {
                for (int x = 0; x < number_of_pixels_in_one_line * 4; x += 4)
                {
                    byte blue = mem_ptr[ index + x];
                    byte green = mem_ptr[ index + x + 1];
                    byte red = mem_ptr[ index + x + 2];
                    byte alpha = mem_ptr[ index + x + 3];

                    int average = (red + green + blue) / 3;
                    if (average > threshold)
                        continue;

                    if (alpha == 0)
                        continue;

                    mem_ptr[index + x + 3]--; //decrease the alpha value
                    finished_working_flag = false;

                }
                index+= number_of_pixels_in_one_line * 4;
            }

            _3D.STATE_bitmap_end_direct_memory_access(bmp, 0);

            return (finished_working_flag);
        }


        
        
        
        unsafe bool add_texture_exterior(uint bmp, byte threshold)
        {
            bool finished_working_flag = true;

            byte* mem_ptr = null;
            int number_of_bytes_per_pixel;
            int number_of_pixels_in_one_line;
            int number_of_lines;

            _3D.STATE_bitmap_start_direct_memory_access(bmp, 0, &mem_ptr, out number_of_bytes_per_pixel, out number_of_pixels_in_one_line, out  number_of_lines);
            if (mem_ptr == null)
            {
                //error_msg();
                return (true);
            }

            if (number_of_bytes_per_pixel != 4) //safty check
                return (true); //Since we called _3D.STATE_bitmap_set_default_format(BITMAP_FORMAT_A8R8G8B8);
            //at the beginning of the program, all non-transparent bitmaps must have been converted to 32 bit format


            int index = 0;
            for (int line = 0; line < number_of_lines; line++)
            {
                for (int x = 0; x < number_of_pixels_in_one_line * 4; x += 4)
                {
                    byte blue = mem_ptr[ index + x];
                    byte green = mem_ptr[ index + x + 1];
                    byte red = mem_ptr[ index + x + 2];
                    byte alpha = mem_ptr[ index + x + 3];

                    int average = (red + green + blue) / 3;
                    if (average > threshold)
                        continue;

                    if (alpha == 255)
                        continue;

                    mem_ptr[index + x + 3]++; //increase the alpha value
                    finished_working_flag = false;

                }
                index += number_of_pixels_in_one_line * 4;
            }

            _3D.STATE_bitmap_end_direct_memory_access(bmp, 0);

            return (finished_working_flag);
        }


        enum Modes { Increase_Alpha, Decrease_Alpha};

        Modes m_mode = Modes.Decrease_Alpha;
        int m_delay_count = 0;
 
 void do_logo_alpha_transparency_effect(uint m_logo_bmp)
{
	if(m_delay_count>0) {
		m_delay_count--;
		return;
	}

    
    if (m_mode == Modes.Decrease_Alpha)
    {
		bool finished_flag=remove_texture_exterior(m_logo_bmp, 200);
		if(finished_flag) {
            m_mode = Modes.Increase_Alpha;
			m_delay_count=1000;
		}
		return;
	}


	bool finished_flag2=add_texture_exterior(m_logo_bmp, 200);
	if(finished_flag2) {
        m_mode = Modes.Decrease_Alpha;
        m_delay_count = 1000;
    }

 }


   
    }
}