﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using _3dstate;
using Math3D;


/*
 * This is a continuation of Helow_3D_World
 * The added code makes this sample a better candidate for being a starting point template for many different applications
 * You can use this sample as a starting point for your application
 * Or you can use it to easily add some 3D to your application
 * 
 */ 

namespace Hello_3D_World3
{
    public partial class Form1 : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        double STEP = 3;
        double ANGLES = 0.25;


        public Form1()
        {
            InitializeComponent();
        }

        bool load_world(string world_file_name)
        {
           
            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return(false);
            }

            //next we will set the varialbe STEP
            //this is needed for two things:
            //1) When we move forward inside the world we advance STEP units each time
            //   so we need to set a value here which will fit to the world
            //   in a world that its with is millions of virtual units STEP might be 100 for example
            //   in a world where the entire width of the world is 10 virtual units, STEP will be very small (0.01 for example)
            //
            // 2) We also set _3D.STATE_camera_set_distance_from_eye()
            //    for more information read in _3d.cs

            double[,] world_bounding_box = new double[2, 3];
            _3D.STATE_group_get_bounding_box(0, world_bounding_box);

            double x_width = world_bounding_box[1, 0] - world_bounding_box[0, 0];
            double y_width = world_bounding_box[1, 1] - world_bounding_box[0, 1];
            double max_side = Math.Max(x_width, y_width);
            STEP = max_side / 40000;

            uint camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_distance_from_eye(camera, STEP/10); //also sets the Z buffer resolution

            return (true);


        }

        private void Form1_Load(object sender, EventArgs e)
        {
            load_world("..\\..\\..\\..\\worlds\\flat_ground\\flat_ground.wld");
            
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;


            MessageBox.Show("Use the arrow keys to move.\r\n\nkeys A,Z while moving forward to ascend/descend\r\n\nKey Q to fly horizontally", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
     

            ActiveControl = pictureBox_rendering_window;

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
            //_3D.STATE_engine_close(); //not needed anymore

        }

        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            move_camera_according_to_keyboard_input();

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            
            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }

        void move_camera_according_to_keyboard_input()
        {
            double[] location = new double[3];
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_get_location1(camera, location);


            double[] direction = new double[3];
            _3D.STATE_camera_get_direction1(camera, direction);
            Vector.mul_vec(direction, direction, STEP);

            bool move_flag = false;

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                Vector.add_vec(location, direction);
                move_flag = true;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                Vector.sub_vec(location, direction);
                move_flag = true;
            }


            if (move_flag == true)
            {
                if(checkBox_collision.Checked)
                    _3D.STATE_camera_move_with_collision_detection(camera, location, STEP * 10);
                else
                    _3D.STATE_camera_set_location1(camera, location);

            }


            //turn left and right world space
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
                _3D.STATE_camera_rotate_z(camera, ANGLES, _3D.WORLD_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
                _3D.STATE_camera_rotate_z(camera, -ANGLES, _3D.WORLD_SPACE);



            //----------

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.A) < 0)
                _3D.STATE_camera_rotate_y(camera, ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Z) < 0)
                _3D.STATE_camera_rotate_y(camera, -ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Q) < 0)
                _3D.STATE_camera_set_tilt(camera, 0);


        }



    }
}
