﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;


namespace Tank
{
    public partial class Tank : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        uint m_background_music;
        uint m_tank_engine_sound;
        uint m_ariel18;
        uint m_tank_object;
        double[,] m_world_bounding_box = new double[2, 3];

        double m_fog_visible_distance = 500000;
        double m_fog_max_visible_distance = 1e8;
        double m_fog_min_visible_distance = 1e5;


        double m_tank_max_speed = 150;
        double m_tank_min_speed = -80;
        double m_tank_speed = 50;

        uint m_camera;
        double m_camera_min_height = 400; ////height above the ground

        double m_tank_height;


       
        public Tank()
        {
            InitializeComponent();
        }

        bool load_world(string world_file_name)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }


            //double[,] world_bounding_box = new double[2, 3];
            _3D.STATE_group_get_bounding_box(0, m_world_bounding_box);

            double x_width = m_world_bounding_box[1, 0] - m_world_bounding_box[0, 0];
            double y_width = m_world_bounding_box[1, 1] - m_world_bounding_box[0, 1];
            double max_side = Math.Max(x_width, y_width);

            m_camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_distance_from_eye(m_camera, max_side / 40000);

            //Put some ground details texture
            uint ground_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\terrain1\\Bitmaps\\second_bitmap.bmp", -1);
            if (ground_bmp != 0)
            {
                //Try cancel this line or putting a different number than 250 and see what it does
                _3D.STATE_group_wrap_a_bitmap(0, ground_bmp, 250, 250, 1);
            }



            add_some_light();

            return (true);






        }

        private void Tank_Load(object sender, EventArgs e)
        {
            m_ariel18 = _3D.STATE_font_create("Ariel", 18, _3D.FONT_CREATE_DEFAULT);

            
            //Set Background music
            m_background_music = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Elgar.mp3", 0, _3D.SOUND_DISTANCE_DEFAULT);
            _3D.STATE_sound_play(m_background_music, _3D.SOUND_LOOP_NORMAL);

            m_explosion_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\Explosion1.wav", 0, _3D.SOUND_DISTANCE_DEFAULT);



            //load_world and models
            load_world("..\\..\\..\\..\\worlds\\terrain1\\terrain1.3dstate");
            load_tank("..\\..\\..\\..\\models\\tanks\\t72bm-tank_des.3dstate");
            load_bomb("..\\..\\..\\..\\models\\bombs\\oildrum.3dstate");
            load_explosion("..\\..\\..\\..\\models\\effects\\explosion.wld");





            //Just one line is needed to add the shadows to the tank !!!
            _3D.STATE_object_auto_add_shadows(m_tank_object, _3D.ON, 1);

            //Set the sound of the wind/tank
            m_tank_engine_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\propeller_sound.wav", m_tank_object, 600000);
            _3D.STATE_sound_set_volume(m_tank_engine_sound, 800);
            _3D.STATE_sound_play(m_tank_engine_sound, _3D.SOUND_LOOP_NORMAL);


            _3D.STATE_engine_show_frames_per_second_rate(_3D.INFORMATION_ALL);

            make_the_camera_follow_the_tank_automatically();

            _3D.STATE_engine_set_fog(_3D.ON, 220, 220, 180, _3D.FOG_MODE_AUTO, m_fog_visible_distance, _3D.FOG_START_DISTANCE_DEFAULT, 0);

            show_help_popup();

            
            //add our OnIdle function to the application on idle event
            //see more details at the Onidle function in this page
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;



            ActiveControl = pictureBox_rendering_window;

        }

        bool load_tank(string file_name)
        {
            //load the tank
            m_tank_object = _3D.STATE_object_create_from_file(file_name);
            if (m_tank_object == 0)
            {
                MessageBox.Show("Failed loading tank model.\r\nFile=" + file_name, "Failed loading tank model", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return (false); //safty
            }

            //The object is turned around so lets turn it 180 degrees
            //This is important so that the _3D.STATE_chase functions work as
            //we expect them to work.
            //should come immediatelly after we load the object
            _3D.STATE_object_rotate_z(m_tank_object, 180, _3D.WORLD_SPACE);
            _3D.STATE_object_redefine_axis_system(m_tank_object);

            
            
            
            //double[] scale = { 10, 10, 10 }; //make the tank bigger
            //_3D.STATE_object_set_scale(m_tank_object, scale);


            double length, width;
            _3D.STATE_object_get_dimensions(m_tank_object, out length, out width, out m_tank_height);
           


            return (true);

        }


        private void Tank_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
            //_3D.STATE_engine_close(); //no need to call. faster to exit when not calling

        }

        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            check_whether_a_bomb_is_fired();
            check_whether_a_bomb_has_to_explode();

            change_settings_according_to_keyboard_input();

            move_tank();

            make_sure_we_get_more_OnIdle_events();

        }


        void make_sure_we_get_more_OnIdle_events()
        {
            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...
        }



    }
}
