﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;


using _3dstate;
using Math3D;



namespace Terrain2
{
    public partial class Terrain2 : Form
    {
        uint m_camera;

        double ANGLES = 5;
        double STEP = 3000;



        //uint m_terrain_bitmap = 0;
        //double m_min_height = 1000;
        //double m_max_height = 30000;

        private EventHandler m_idleEventHandler;
        bool m_exiting = false;

        
        
        public Terrain2()
        {
            InitializeComponent();
        }

        bool intialize_3d_engine()
        {

            int rc = _3D.STATE_engine_load_world("..\\..\\..\\..\\worlds\\flat_ground\\flat_ground2.wld", "", "", _3D.EDITOR_MODE);
            if (rc == _3D.VR_ERROR)
            {
                MessageBox.Show("failed loading world");
                return (false);
            }

            m_camera = _3D.STATE_camera_get_default_camera();




            return (true);


        }

        
        private void Terrain2_Load(object sender, EventArgs e)
        {
            intialize_3d_engine();
        }

        private void Terrain2_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
        }

        private void button_create_terrain_Click(object sender, EventArgs e)
        {
            OpenFileDialog fd = new OpenFileDialog();

            fd.Title = "Select the elevation data source";

            DialogResult res = fd.ShowDialog();

            if (res != DialogResult.OK)
                return;


            if (String.IsNullOrEmpty(fd.FileName))
                return;


            string elevation_file = fd.FileName;

            fd.Title = "Select the Satellite image";

            res = fd.ShowDialog();

            if (res != DialogResult.OK)
                return;


            if (String.IsNullOrEmpty(fd.FileName))
                return;

            string image_file = fd.FileName;


            int orig_image_width, orige_image_height;
            double  dem_height;
            double[] lowest_point = new double[3];
            double[] highest_point = new double[3];

            string temp_files_folder = "Temp Files";
            if (Directory.Exists(temp_files_folder) == false)
                Directory.CreateDirectory(temp_files_folder);


            _3D.STATE_engine_create_terrain2(elevation_file, image_file, new double[3] { 1000000, 1000000, 120000 }, _3D.NO, 1024, _3D.NO, 30000, _3D.NO, 0, temp_files_folder, out orig_image_width, out orige_image_height, out  dem_height, lowest_point, highest_point, "..//..//..//..//Models//DEM//Haifa//hef_tif_8bit_50percent.tfw", null);

            m_camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_location(m_camera, -15000 , -15000, 60000);
            _3D.STATE_camera_set_direction(m_camera, 1, 1, -0.3);

            if (checkBox_save.Checked)
            {
                string save_folder = "Save";
                if (Directory.Exists(save_folder) == false)
                    Directory.CreateDirectory(save_folder);


                string file_name = save_folder + "\\terrain.3dstate";
                //save the file so you can load it in other applications.
                _3D.STATE_engine_save(file_name, 0, 0);

                string full_path = Path.GetFullPath(file_name);
                MessageBox.Show("The world was saved at: " + full_path, "The world was saved !", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }


            checkBox_collision.Visible = true;
            //button_shapefile.Visible = true;
            
            
            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;
        
            


        }


        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {

            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            move_camera_according_to_keyboard_input();

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);




            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }

        void move_camera_according_to_keyboard_input()
        {
            bool arrow_keys_pressed = false;

            double[] location = new double[3];
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_get_location1(camera, location);

            //if(GetAsyncKeyState(VK_UP)<0)	STATE_camera_move(camera, CAMERA_SPACE, -STEP,0,0);
            //if(GetAsyncKeyState(VK_DOWN)<0) STATE_camera_move(camera, CAMERA_SPACE,  STEP,0,0);

            double[] direction = new double[3];
            _3D.STATE_camera_get_direction1(camera, direction);
            Vector.mul_vec(direction, direction, STEP);

            bool move_flag = false;
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                Vector.add_vec(location, direction);
                move_flag = true;
                arrow_keys_pressed = true;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                Vector.sub_vec(location, direction);
                move_flag = true;
                arrow_keys_pressed = true;
            }


            if (move_flag == true)
            {
                if(checkBox_collision.Checked)
                    _3D.STATE_camera_move_with_collision_detection(camera, location, STEP * 10);
                else
                    _3D.STATE_camera_set_location1(camera, location);

            }


            //turn left and right world space
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
            {
                _3D.STATE_camera_rotate_z(camera, ANGLES, _3D.WORLD_SPACE);
                arrow_keys_pressed = true;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
            {
                _3D.STATE_camera_rotate_z(camera, -ANGLES, _3D.WORLD_SPACE);
                arrow_keys_pressed = true;
            }


            //up and down
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.H) < 0)
                _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, STEP);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.N) < 0)
                _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, -STEP);


            //----------

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.A) < 0)
                _3D.STATE_camera_rotate_y(camera, ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Z) < 0)
                _3D.STATE_camera_rotate_y(camera, -ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Q) < 0)
                _3D.STATE_camera_set_tilt(camera, 0);


            if (arrow_keys_pressed)
            {
                ActiveControl = pictureBox_rendering_window;
            }

        }

        private void button_help_Click(object sender, EventArgs e)
        {
            string str = File.ReadAllText("..\\..\\..\\..\\Docs\\terrain2_help.txt");

            if(String.IsNullOrEmpty(str))
                str="couldnt find help file terrain2_help.txt";

            MessageBox.Show(str, "Istructions", MessageBoxButtons.OK, MessageBoxIcon.Information);

        }

        private void button_shapefile_Click(object sender, EventArgs e)
        {
            _3D.STATE_object_create_objects_from_shapefile_records("..\\..\\..\\..\\Models\\DEM\\GIS Samples\\Haifa height\\hifa_bld.shp", _3D.NO, 0,0,0, 1000, null, null);
        }


    }
}
