﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using _3dstate;
using Math3D;


namespace TrainYard
{
    public partial class TrainYard : Form
    {
        private EventHandler m_idleEventHandler;
        bool m_exiting = false;
        double m_walking_step = 3;
        double m_angles_to_turn = 1;

        uint m_shooting_mark_bmp;
        uint m_shoot_sound;

        uint m_camera;
        uint m_crosshair_layer;
        
        public TrainYard()
        {
            InitializeComponent();
        }

        private void TrainYard_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            Application.Idle -= m_idleEventHandler;
        }

        private void TrainYard_Load(object sender, EventArgs e)
        {
            load_world("..\\..\\..\\..\\worlds\\TrainYard\\TrainYard.3dstate");

            m_camera = _3D.STATE_camera_get_default_camera();


            m_shooting_mark_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\TrainYard\\extra_bitmaps\\smark", 0);

            
            uint crosshair_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\TrainYard\\extra_bitmaps\\crosshair", 0);
            m_crosshair_layer=_3D.STATE_layer_create(45, 45, 55, 55, crosshair_bmp, _3D.CONTROL_GLASS);

            m_shoot_sound = _3D.STATE_sound_load("..\\..\\..\\..\\sound\\shoot.wav", 0, _3D.SOUND_DISTANCE_DEFAULT);

            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;

            MessageBox.Show("Use the arrow keys to move.\r\n\nPress the Space Bar to shoot (makes a mark)\r\n\r\nClick with the mouse on the 3D window to put a shooting mark\r\n\r\nThis demo demonstrates how to use the mouse to interact with the 3D world and how to get the polygon and location for a specific point on the 2D window ", "Instructions", MessageBoxButtons.OK, MessageBoxIcon.Information);
            ActiveControl = pictureBox_rendering_window;
        }

        bool load_world(string world_file_name)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world(world_file_name, "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }


            return (true);


        }

        private void OnIdle(object sender, EventArgs e)
        {
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            move_camera_according_to_keyboard_input();

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Space) < 0)
                add_shooting_mark();


            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);


            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }


        void move_camera2(double step)
        {
            double[] direction = new double[3];
            _3D.STATE_camera_get_direction1(m_camera, direction);

            Vector.mul_vec(direction, -step);

            double[] current_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, current_location);

            double[] wanted_location = Vector.copy(current_location);
            Vector.add_vec(wanted_location,direction);

            //now that we know the wanted location lets move the camera with collision detection
            _3D.STATE_camera_move_with_collision_detection(m_camera, wanted_location, m_walking_step * 10);

        }


        //notice also move_camera2 that does the same just
        //with a different approach
        void move_camera(double step)
        {
            double[] current_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, current_location);

            //move the camera so we can easily get the location where we want to go
            _3D.STATE_camera_move(m_camera, _3D.CAMERA_SPACE, step, 0, 0);
            double[] wanted_location = new double[3];
            _3D.STATE_camera_get_location1(m_camera, wanted_location);

            //we moved the camera but it could be that the way was blocked
            //so let move the camera back to it previous location
            //and do the movement again with collision detection
            _3D.STATE_camera_set_location1(m_camera, current_location); //return the camera back
            
            //now that we know the wanted location lets move the camera with collision detection
            _3D.STATE_camera_move_with_collision_detection(m_camera, wanted_location, m_walking_step * 10 );

        }

        void move_camera_according_to_keyboard_input()
        {
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
                _3D.STATE_camera_rotate_z(m_camera, m_angles_to_turn, _3D.WORLD_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
                _3D.STATE_camera_rotate_z(m_camera, -m_angles_to_turn, _3D.WORLD_SPACE);


            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
                move_camera(-m_walking_step);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
                move_camera(m_walking_step);


        }
        



        DateTime m_last_shooting = DateTime.Now;


        void add_shooting_mark()
        {
            TimeSpan ts = DateTime.Now - m_last_shooting;
            if (ts.TotalSeconds < 1)
                return;
                 
            int width = _3D.STATE_engine_get_rendering_window_width();
            int height = _3D.STATE_engine_get_rendering_window_height();

            uint obj, poly;
            double[] xyz = new double[3];
            _3D.STATE_engine_2D_point_to_3D(width/2, height/2, xyz, out obj, out poly);

            if (poly == 0)
                return;


            int[] rgb = { 255, 255, 255 };
            double[] direction = { 0, 0, 1 };
            uint patch=_3D.STATE_polygon_add_patch_easy(poly, xyz, 5, direction, m_shooting_mark_bmp, rgb);

            _3D.STATE_polygon_set_translucent(patch, _3D.DARK_FILTER_GLASS);
            _3D.STATE_polygon_set_second_bitmap(patch, 0);

            //_3D.STATE_layer_create(0, 0, 20, 20, m_shooting_mark_bmp, 0);

            _3D.STATE_sound_play(m_shoot_sound, _3D.SOUND_NO_LOOP);


            m_last_shooting = DateTime.Now;

        }

        private void pictureBox_rendering_window_MouseClick(object sender, MouseEventArgs e)
        {
            uint obj, poly;
            double[] xyz = new double[3];
            _3D.STATE_engine_2D_point_to_3D(e.X, e.Y, xyz, out obj, out poly);

            if (poly == 0)
            {
                MessageBox.Show("no polygon here");
                return;
            }

            byte[] rgb = { 255, 128, 128 };
            _3D.STATE_polygon_set_light(poly, rgb);
            
        }



    }
}
