﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;

using _3dstate;

/*
 *  A simple viewer to view 3d models
 * 
 */ 


namespace Viewer
{
    public partial class Form1 : Form
    {
        Timer m_timer;
        string[] m_3dfile_types = {"*.3dstate", "*.morfit", "*.state", "*.wld", "*.x" };
        uint m_3d_obj=0;
        uint m_light;
        bool no_object_was_loaded = true;

        double m_camera_min_distance_to_object;
        double m_camera_max_distance_to_object;
        double m_camera_distance_to_object;
        double m_step; //the step when zooming in and out on an object

        
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            fill_list_box_with_files("..\\..\\..\\..\\models", m_3dfile_types, checkBox_sub_folders.Checked);
        }

        void init_timer()
        {
            m_timer = new Timer();
            m_timer.Interval = 1;
            m_timer.Tick += new EventHandler(timer_Tick);
            m_timer.Start();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if(m_timer!=null)
                m_timer.Stop(); 
        }

        void handle_mouse_dragging()
        {
            double[] object_location = new double[] { 0, 0, 0 };

            
            uint camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_move_around_point_according_to_mouse_position(camera, object_location, m_camera_distance_to_object, (int)Keys.LButton, _3D.NO);

            //move the camera closer or further away if the arrow keys are pressed
            if ((_3D.GetAsyncKeyState(Keys.A) < 0) && m_camera_distance_to_object > m_camera_min_distance_to_object)
                m_camera_distance_to_object -= m_step;

            if ((_3D.GetAsyncKeyState(Keys.Z) < 0) && m_camera_distance_to_object < m_camera_max_distance_to_object)
                m_camera_distance_to_object += m_step;


        }


        //alternetivly we can use the Application.Idle event or a separate thread
        void timer_Tick(object sender, EventArgs e)
        {

            handle_mouse_dragging();

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);


            //lets turn the object (the object is in the origin)
            if (m_3d_obj == 0)
                return;


            if (checkBox_auto_turn.Checked)
                _3D.STATE_object_rotate_z(m_3d_obj, 0.4, _3D.OBJECT_SPACE);
            

        }

        private void button_browse_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();

            string dir = Directory.GetCurrentDirectory();

            dlg.SelectedPath =  "..\\..\\..\\..\\models";
            //dlg.Filter = "3D files (*.3dstate)|*.3dstate|(*.morfit)|*.morfit|(*.state)|*.state|(*.wld)|*.wld|(*.x)|*.x|(*.xyz)|*.xyz|All files (*.*)|*.*";
            

            DialogResult res = dlg.ShowDialog();
            if (res != DialogResult.OK)
                return;

            if (String.IsNullOrEmpty(dlg.SelectedPath))
                return;

           
            fill_list_box_with_files(dlg.SelectedPath, m_3dfile_types, checkBox_sub_folders.Checked);

        }

        int fill_list_box_with_files(string folder, string[] file_types, bool include_sub_directories)
        {
            textBox_folder.Text = folder;

            SearchOption search_option = SearchOption.AllDirectories;
            if (include_sub_directories == false)
                search_option = SearchOption.TopDirectoryOnly;

            
            List<string> all_3d_files = get_all_filenames_in_folder(folder, file_types, search_option);

            listBox_models.Items.Clear();

            foreach (string name in all_3d_files)
                listBox_models.Items.Add(name);


            if (listBox_models.Items.Count > 0)
                label_select_object.Visible = true;
            else
                label_select_object.Visible = false;
            
            return (all_3d_files.Count);

        }

        List<string> get_all_filenames_in_folder(string folder, string[] filters, SearchOption search_option)
        {
            List<string> filenames=new List<string>();
            
            foreach (string filter in filters)
            {
                string[] names=Directory.GetFiles(folder,filter, search_option);
                foreach(string name in names)
                    filenames.Add(name);

            
            }

            return(filenames);

        }

        private void listBox_models_SelectedIndexChanged(object sender, EventArgs e)
        {
            object obj=listBox_models.SelectedItem;
            
            if (obj == null)
                return;

            load_object(obj.ToString());

            label_instructions.Visible = true;

        }

        void load_object(string file_name)
        {
            if (m_3d_obj != 0)
            {
                _3D.STATE_object_delete(m_3d_obj);
                m_3d_obj = 0;
            }

            m_3d_obj = _3D.STATE_object_create_from_file(file_name);
            if (m_3d_obj == 0)
            {
                MessageBox.Show("Failed to load model: \r\n" + file_name, "Failed to load model");
                return;
            }

            
            if (no_object_was_loaded)
            {
                //we are now going to make the first render.
                //many functions such as STATE_object_get_bounding_box() dont work before we make at least one render
                IntPtr window_handle = pictureBox_rendering_window.Handle;
                _3D.STATE_engine_render(window_handle, 0);
            }


            //make sure that whether objects are very small or very big they will be put in an appropriate distance from the camera so that they are seen correctly
            double width, height, length;
            _3D.STATE_object_get_dimensions(m_3d_obj, out length, out width, out height);

            double max=Math.Max(height, length);
            max = Math.Max(max, width);

            if(max<=0) {
                MessageBox.Show("Model volume is 0.\r\nModel=\r\n" + file_name, "Model volume is 0" );
                return;
                //max = 100;
            }

            m_camera_min_distance_to_object = max / 4;
            m_camera_max_distance_to_object = max * 4;
            m_camera_distance_to_object = max;
            m_step = max / 25;

            uint camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_set_location(camera, max , 0, 0);
            _3D.STATE_camera_point_at(camera, 0, 0, 0);

            _3D.STATE_camera_set_distance_from_eye(camera, max / 100);



            if (no_object_was_loaded)
                first_time_init();

        }

        void first_time_init()
        {
            _3D.STATE_engine_show_frames_per_second_rate(_3D.INFORMATION_ALL);
            create_light();

            init_timer();

            no_object_was_loaded = false;

        }

        private void checkBox_light_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_light.Checked)
            {
                if (m_light == 0)
                    create_light();

                _3D.STATE_light2_disable_all_lighting(false);
                _3D.STATE_light2_enable(m_light);
                return;
            }

            _3D.STATE_light2_disable(m_light);
            _3D.STATE_light2_disable_all_lighting(true);



        }

        void create_light()
        {
            double[] direction = { -1, -1, -1 };
            m_light = _3D.STATE_light2_create_directional_light("my_light", direction);
        }

        private void button_help_Click(object sender, EventArgs e)
        {
            MessageBox.Show("This viewer will show you all the 3D models in the selected folder and sub folders (if checked).\r\n\r\n1) First select a folder (by clicking on the select folder button)\r\n\r\n2) Click on one of the 3d models showing at the list appearing at the right\r\n\r\n3) Use the Keys A,Z to zoom in and out. You can also drag the object with the mouse\r\n\r\nEnjoy !", "Help", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }




    }
}
