﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;


using _3dstate;
using Math3D;

namespace terrain
{
    public partial class terrain : Form
    {
        uint m_camera;
        uint m_light;

        double ANGLES = 3;

        bool m_thread_active = false;
        Thread m_thread = null;

        uint m_terrain_grp;


        uint m_terrain_bitmap=0;
        double m_min_height = 0;
        double m_max_height = 200;
        double m_1x1_size = 10;
        double STEP = 1;




        private EventHandler m_idleEventHandler;
        bool m_exiting = false;


        
        public terrain()
        {
            InitializeComponent();
        }

        private void terrain_Load(object sender, EventArgs e)
        {
            intialize_3d_engine();


            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;


        }

        bool intialize_3d_engine()
        {

           int rc= _3D.STATE_engine_load_world("..\\..\\..\\..\\worlds\\flat_ground\\flat_ground2.wld", "", "", _3D.EDITOR_MODE);
           if (rc == _3D.VR_ERROR)
           {
               MessageBox.Show("failed loading world");
               return(false);
           }

            m_camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_set_location(m_camera, -m_max_height/2, -m_max_height/2, m_max_height);
            _3D.STATE_camera_set_direction(m_camera, 1, 1, -0.3);

            _3D.STATE_camera_set_distance_from_eye(m_camera, m_1x1_size / 10);

            return (true);


        }

        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface.
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {
            if (m_thread_active )
                return;
            
            if (m_exiting)
                return; //so we wont get here after Form_close event

            if (_3D.STATE_engine_is_engine_empty() == _3D.YES)
                return;

            move_camera_according_to_keyboard_input();

            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0);

            if( (m_thread_active==false) && (button_create_terrain.Text =="Stop"))
                button_create_terrain.Text = "Create Terrain";



            //The invalidate causes an onpaint of the form which when finishes calls onidle again.            
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);
            //for some reasons, only if we have OnPaint event of the form
            //OnIdle is getting called. So here we invalidate the form
            //with a zero space rectangle and with invalidate_children==false
            //so that OnIdle gets called again.
            //Application.RaiseIdle(null) wont work since it will cause a stack overflow by calling onidle recursively ...


        }

        void move_camera_according_to_keyboard_input()
        {
            bool arrow_keys_pressed = false;

            double[] location = new double[3];
            uint camera = _3D.STATE_camera_get_default_camera();

            _3D.STATE_camera_get_location1(camera, location);

            //if(GetAsyncKeyState(VK_UP)<0)	STATE_camera_move(camera, CAMERA_SPACE, -STEP,0,0);
            //if(GetAsyncKeyState(VK_DOWN)<0) STATE_camera_move(camera, CAMERA_SPACE,  STEP,0,0);

            double[] direction = new double[3];
            _3D.STATE_camera_get_direction1(camera, direction);
            Vector.mul_vec(direction, direction, STEP);

            bool move_flag = false;
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Up) < 0)
            {
                Vector.add_vec(location, direction);
                move_flag = true;
                arrow_keys_pressed = true;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Down) < 0)
            {
                Vector.sub_vec(location, direction);
                move_flag = true;
                arrow_keys_pressed = true;
            }


            if (move_flag == true)
            {
                if(checkBox_collision.Checked)
                    _3D.STATE_camera_move_with_collision_detection(camera, location, STEP * 10);
                else
                    _3D.STATE_camera_set_location1(camera, location);

            }


            //turn left and right world space
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Left) < 0)
            {
                _3D.STATE_camera_rotate_z(camera, ANGLES, _3D.WORLD_SPACE);
                arrow_keys_pressed = true;
            }

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Right) < 0)
            {
                _3D.STATE_camera_rotate_z(camera, -ANGLES, _3D.WORLD_SPACE);
                arrow_keys_pressed = true;
            }


            //up and down
            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.H) < 0)
                _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, STEP);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.N) < 0)
                _3D.STATE_camera_move(camera, _3D.WORLD_SPACE, 0, 0, -STEP);


            //----------

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.A) < 0)
                _3D.STATE_camera_rotate_y(camera, ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Z) < 0)
                _3D.STATE_camera_rotate_y(camera, -ANGLES, _3D.CAMERA_SPACE);

            if (_3D.STATE_utilities_get_keyboard_key_state((int)Keys.Q) < 0)
                _3D.STATE_camera_set_tilt(camera, 0);


            if (arrow_keys_pressed)
            {
                ActiveControl = pictureBox_rendering_window;
            }

        }

        void terminate_thread()
        {
            m_thread_active = false;
            Thread.Sleep(100);
            if (m_thread == null)
                return;

            m_thread.Abort();
            m_thread = null;
        }

        private void terrain_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_exiting = true;
            terminate_thread();
        }



        void create_terrain_thread()
        {
            if (m_terrain_grp != 0)
            {
                _3D.STATE_group_delete_members(m_terrain_grp);
                m_terrain_grp = 0;

            }
          
            m_terrain_grp=create_terrain_from_bitmap(m_terrain_bitmap, m_min_height, m_max_height);

            if (checkBox_optimize.Checked)
            {
                int num_of_polygons = _3D.STATE_group_get_number_of_polygons(m_terrain_grp);

                double range = m_max_height - m_min_height;
                uint optimize_grp = _3D.STATE_group_create_reduced_copy(m_terrain_grp, range / 50);

                _3D.STATE_group_delete_members(m_terrain_grp);

                num_of_polygons = _3D.STATE_group_get_number_of_polygons(0);

                m_terrain_grp = optimize_grp;

            }
            
            _3D.STATE_group_wrap_a_bitmap(0, m_terrain_bitmap, 1, 1, 0);

            if (checkBox_ground_texture.Checked)
            {
                //Put some ground details texture
                uint ground_bmp = _3D.STATE_bitmap_load("..\\..\\..\\..\\worlds\\terrain1\\Bitmaps\\second_bitmap.bmp", -1);
                if (ground_bmp != 0)
                {
                    //set the ground texture as the second texture (index 1) and tile it 10x10 accross the terrain
                    _3D.STATE_group_wrap_a_bitmap(0, ground_bmp, 10, 10, 1);
                }
            }


            update_progressbar(1);

            if (m_light == 0)
            {
                double[] direction = { 1, 1, -1 };
                m_light = _3D.STATE_light2_create_directional_light("my_light", direction);
            }


            m_thread_active = false;

        }



        private void button_create_terrain_Click(object sender, EventArgs e)
        {
            if (button_create_terrain.Text == "Stop")
            {
                terminate_thread();
                return;
            }
            
            OpenFileDialog fd = new OpenFileDialog();

            DialogResult res= fd.ShowDialog();

            if (res != DialogResult.OK)
                return;
            
            
            if (String.IsNullOrEmpty(fd.FileName))
                return;



            string path = Path.GetDirectoryName(fd.FileName);
            string resized_filename = path +  "\\resized.bmp";
            
            _3D.STATE_utilities_resample_bitmap(fd.FileName, resized_filename, 256, 256);

            m_terrain_bitmap = _3D.STATE_bitmap_load(resized_filename, -1);

            button_create_terrain.Text = "Stop";
            
            m_thread_active = true;
            ThreadStart create_terrain_funtion = new ThreadStart(create_terrain_thread);
            m_thread = new Thread(create_terrain_funtion);
            m_thread.Start();


        }

        void add_some_light()
        {
            //Add a light source
            double[] location = { 0, 0, 1000000 }; //put it up in the sky like the sun
            uint light = _3D.STATE_light_create("light1", location);
            _3D.STATE_light_set_type_of_light(light, _3D.LIGHT_AMBIENT | _3D.LIGHT_DIFFUSE);
            _3D.STATE_light_set_ambient(light, 0.8);
            _3D.STATE_light_set_specular(light, 0);
            _3D.STATE_light_set_diffuse(light, 3);
            _3D.STATE_light_activate(light, _3D.LIGHT_SET);

            location[2] = 10000000; //this height will look better with the plane shadows since is simulate the sun and the sun is very high above ...
            _3D.STATE_light_set_location(light, location);


        }

        public delegate void UpdateProgressBarDelegate(double worked_finished);

        void update_progressbar(double worked_finished)
        {
            if (m_exiting)
                return;

            UpdateProgressBarDelegate update_function_delegate = new UpdateProgressBarDelegate(update_progressbar_internal);

            object[] parameters = { worked_finished };

            Invoke(update_function_delegate, parameters);
        }

        void update_progressbar_internal(double worked_finished)
        {
            double max = progressBar1.Maximum;

            int progress = (int)(max * worked_finished);

            progressBar1.Value = progress;

            if (checkBox_render.Checked)
            {
                IntPtr window_handle = pictureBox_rendering_window.Handle;             
                _3D.STATE_engine_render(window_handle, m_camera);
            }
        }



        uint create_terrain_from_bitmap(uint bmp, double min_height, double max_height)
        {
            int bmp_height = _3D.STATE_bitmap_get_height(bmp);
            int bmp_width = _3D.STATE_bitmap_get_width(bmp);

            //int bmp_height = 110;
            //int bmp_width = 148;

            uint m_terrain_grp = _3D.STATE_group_create("terrain_group");


            for (int y = 0; y < bmp_height-1; y++)
            {
                update_progressbar( (double)y / (double)bmp_height);
                
                for (int x = 0; x < bmp_width-1; x++)
                {
                    if (m_thread_active == false)
                        return(m_terrain_grp);
                    
                    double[] pixel_height = new double[4];
                    
                    byte[] rgba = new byte[4];

                    _3D.STATE_bitmap_get_pixel(bmp, x, y, rgba);
                    pixel_height[0] = convert_rgba_into_height(rgba[0], min_height, max_height);

                    _3D.STATE_bitmap_get_pixel(bmp, x+1, y, rgba);
                    pixel_height[1] = convert_rgba_into_height(rgba[0], min_height, max_height);

                    _3D.STATE_bitmap_get_pixel(bmp, x , y+1, rgba);
                    pixel_height[2] = convert_rgba_into_height(rgba[0], min_height, max_height);

                    _3D.STATE_bitmap_get_pixel(bmp, x+1, y + 1, rgba);
                    pixel_height[3] = convert_rgba_into_height(rgba[0], min_height, max_height);

                    create_2_triangles(y * m_1x1_size, x * m_1x1_size, pixel_height, m_terrain_grp);
                
                }
            }


            return (m_terrain_grp);
        }

        double convert_rgba_into_height(byte color, double min_height, double max_height)
        {
            double factor = (double)color / 255;

            double haight_range = max_height - min_height;

            return (haight_range * factor + min_height);

        }

        void create_2_triangles(double x, double y, double[] pixel_height, uint m_terrain_grp)
        {
            uint poly=_3D.STATE_polygon_create();

            double[] point=new double[6];
            point[0]=x;
            point[1]=y;
            point[2]= pixel_height[0];
            _3D.STATE_polygon_add_point(poly, point);

            point[0] = x+m_1x1_size;
            point[1] = y+m_1x1_size;
            point[2] = pixel_height[3];
            _3D.STATE_polygon_add_point(poly, point);

            point[0] = x + m_1x1_size;
            point[1] = y ;
            point[2] = pixel_height[2];
            _3D.STATE_polygon_add_point(poly, point);

            _3D.STATE_polygon_set_color_fill(poly, 0, 255, 0);

            int rc2=_3D.STATE_engine_add_polygon(poly);
            if (rc2 != _3D.OK)
            {
                MessageBox.Show("bad_poly");
            }

            _3D.STATE_polygon_set_group(poly, m_terrain_grp);


            uint poly2 = _3D.STATE_polygon_create();

            point[0] = x;
            point[1] = y;
            point[2] = pixel_height[0];
            _3D.STATE_polygon_add_point(poly2, point);

            point[0] = x ;
            point[1] = y + m_1x1_size;
            point[2] = pixel_height[1];
            _3D.STATE_polygon_add_point(poly2, point);

            point[0] = x + m_1x1_size;
            point[1] = y + m_1x1_size;
            point[2] = pixel_height[3];
            _3D.STATE_polygon_add_point(poly2, point);

            _3D.STATE_polygon_set_color_fill(poly2, 255, 0, 0);

            int rc= _3D.STATE_engine_add_polygon(poly2);
            if (rc != _3D.OK)
            {
                MessageBox.Show("bad_poly");
            }

            _3D.STATE_polygon_set_group(poly2, m_terrain_grp);


        }

        private void button_help_Click(object sender, EventArgs e)
        {
            string str = File.ReadAllText("..\\..\\..\\..\\Docs\\terrain1_help.txt");

            if (String.IsNullOrEmpty(str))
                str = "couldnt find help file terrain1_help.txt";

            MessageBox.Show(str, "Istructions", MessageBoxButtons.OK, MessageBoxIcon.Information);

        }


    }
}
